package java.awt;

import sun.awt.image.SunWritableRaster;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import sun.awt.image.ByteInterleavedRaster;
import sun.awt.image.IntegerInterleavedRaster;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.lang.ref.WeakReference;
import java.awt.image.WritableRaster;
import java.awt.image.ColorModel;

abstract class TexturePaintContext implements PaintContext
{
    public static ColorModel xrgbmodel;
    public static ColorModel argbmodel;
    ColorModel colorModel;
    int bWidth;
    int bHeight;
    int maxWidth;
    WritableRaster outRas;
    double xOrg;
    double yOrg;
    double incXAcross;
    double incYAcross;
    double incXDown;
    double incYDown;
    int colincx;
    int colincy;
    int colincxerr;
    int colincyerr;
    int rowincx;
    int rowincy;
    int rowincxerr;
    int rowincyerr;
    private static WeakReference<Raster> xrgbRasRef;
    private static WeakReference<Raster> argbRasRef;
    private static WeakReference<Raster> byteRasRef;
    
    public static PaintContext getContext(BufferedImage bufferedImage, AffineTransform affineTransform, RenderingHints renderingHints, Rectangle rectangle) {
        WritableRaster raster = bufferedImage.getRaster();
        ColorModel colorModel = bufferedImage.getColorModel();
        int width = rectangle.width;
        Object value = renderingHints.get(RenderingHints.KEY_INTERPOLATION);
        int n = (value == null) ? (renderingHints.get(RenderingHints.KEY_RENDERING) == RenderingHints.VALUE_RENDER_QUALITY) : (value != RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
        if (raster instanceof IntegerInterleavedRaster && (n == 0 || isFilterableDCM(colorModel))) {
            IntegerInterleavedRaster integerInterleavedRaster = (IntegerInterleavedRaster)raster;
            if (integerInterleavedRaster.getNumDataElements() == 1 && integerInterleavedRaster.getPixelStride() == 1) {
                return new TexturePaintContext.Int(integerInterleavedRaster, colorModel, affineTransform, width, (boolean)n);
            }
        }
        else if ((raster instanceof ByteInterleavedRaster)) {
            ByteInterleavedRaster byteInterleavedRaster = (ByteInterleavedRaster)raster;
            if (byteInterleavedRaster.getNumDataElements() == 1 && byteInterleavedRaster.getPixelStride() == 1) {
                if (!(n == 0)) {
                    if (isFilterableICM(colorModel)) {
                        return new TexturePaintContext.ByteFilter(byteInterleavedRaster, colorModel, affineTransform, width);
                    }
                }
                else {
                    return new TexturePaintContext.Byte(byteInterleavedRaster, colorModel, affineTransform, width);
                }
            }
        }
        return new TexturePaintContext.Any(raster, colorModel, affineTransform, width, (boolean)n);
    }
    
    public static boolean isFilterableICM(ColorModel colorModel) {
        return colorModel instanceof IndexColorModel && ((IndexColorModel)colorModel).getMapSize() <= 256;
    }
    
    public static boolean isFilterableDCM(ColorModel colorModel) {
        if ((colorModel instanceof DirectColorModel)) {
            DirectColorModel directColorModel = (DirectColorModel)colorModel;
            return isMaskOK(directColorModel.getAlphaMask(), true) && isMaskOK(directColorModel.getRedMask(), false) && isMaskOK(directColorModel.getGreenMask(), false) && isMaskOK(directColorModel.getBlueMask(), false);
        }
        return false;
    }
    
    public static boolean isMaskOK(int n, boolean b) {
        return (b && n == 0) || n == 255 || n == 65280 || n == 16711680 || n == -16777216;
    }
    
    public static ColorModel getInternedColorModel(ColorModel colorModel) {
        if (TexturePaintContext.xrgbmodel == colorModel || TexturePaintContext.xrgbmodel.equals(colorModel)) {
            return TexturePaintContext.xrgbmodel;
        }
        if (TexturePaintContext.argbmodel == colorModel || TexturePaintContext.argbmodel.equals(colorModel)) {
            return TexturePaintContext.argbmodel;
        }
        return colorModel;
    }
    
    TexturePaintContext(ColorModel colorModel, AffineTransform inverse, int bWidth, int bHeight, int maxWidth) {
        this.colorModel = getInternedColorModel(colorModel);
        this.bWidth = bWidth;
        this.bHeight = bHeight;
        this.maxWidth = maxWidth;
        try {
            inverse = inverse.createInverse();
        }
        catch (NoninvertibleTransformException ex) {
            inverse.setToScale(0.0, 0.0);
        }
        this.incXAcross = mod(inverse.getScaleX(), (double)bWidth);
        this.incYAcross = mod(inverse.getShearY(), (double)bHeight);
        this.incXDown = mod(inverse.getShearX(), (double)bWidth);
        this.incYDown = mod(inverse.getScaleY(), (double)bHeight);
        this.xOrg = inverse.getTranslateX();
        this.yOrg = inverse.getTranslateY();
        this.colincx = (int)this.incXAcross;
        this.colincy = (int)this.incYAcross;
        this.colincxerr = fractAsInt(this.incXAcross);
        this.colincyerr = fractAsInt(this.incYAcross);
        this.rowincx = (int)this.incXDown;
        this.rowincy = (int)this.incYDown;
        this.rowincxerr = fractAsInt(this.incXDown);
        this.rowincyerr = fractAsInt(this.incYDown);
    }
    
    static int fractAsInt(double n) {
        return (int)(n % 1.0 * 2.147483647E9);
    }
    
    static double mod(double n, double n2) {
        n %= n2;
        if (n < 0.0) {
            n += n2;
            if (n >= n2) {
                n = 0.0;
            }
        }
        return n;
    }
    
    @Override
    public void dispose() {
        dropRaster(this.colorModel, this.outRas);
    }
    
    @Override
    public ColorModel getColorModel() {
        return this.colorModel;
    }
    
    @Override
    public Raster getRaster(int n, int n2, int a, int n3) {
        if (this.outRas == null || this.outRas.getWidth() < a || this.outRas.getHeight() < n3) {
            this.outRas = this.makeRaster((n3 == 1) ? Math.max(a, this.maxWidth) : a, n3);
        }
        double mod = mod(this.xOrg + (double)n * this.incXAcross + (double)n2 * this.incXDown, (double)this.bWidth);
        double mod2 = mod(this.yOrg + (double)n * this.incYAcross + (double)n2 * this.incYDown, (double)this.bHeight);
        this.setRaster((int)mod, (int)mod2, fractAsInt(mod), fractAsInt(mod2), a, n3, this.bWidth, this.bHeight, this.colincx, this.colincxerr, this.colincy, this.colincyerr, this.rowincx, this.rowincxerr, this.rowincy, this.rowincyerr);
        SunWritableRaster.markDirty(this.outRas);
        return this.outRas;
    }
    
    static synchronized WritableRaster makeRaster(ColorModel colorModel, Raster raster, int n, int n2) {
        if (TexturePaintContext.xrgbmodel == colorModel) {
            if (TexturePaintContext.xrgbRasRef != null) {
                WritableRaster writableRaster2 = (WritableRaster)TexturePaintContext.xrgbRasRef.get();
                if (writableRaster2 != null && writableRaster2.getWidth() >= n && writableRaster2.getHeight() >= n2) {
                    TexturePaintContext.xrgbRasRef = null;
                    return writableRaster2;
                }
            }
            if (n <= 32 && n2 <= 32) {
                n2 = (n = 32);
            }
        }
        else if (TexturePaintContext.argbmodel == colorModel) {
            if (TexturePaintContext.argbRasRef != null) {
                WritableRaster writableRaster = (WritableRaster)TexturePaintContext.argbRasRef.get();
                if (writableRaster != null && writableRaster.getWidth() >= n && writableRaster.getHeight() >= n2) {
                    TexturePaintContext.argbRasRef = null;
                    return writableRaster;
                }
            }
            if (n <= 32 && n2 <= 32) {
                n2 = (n = 32);
            }
        }
        if (raster != null) {
            return raster.createCompatibleWritableRaster(n, n2);
        }
        return colorModel.createCompatibleWritableRaster(n, n2);
    }
    
    static synchronized void dropRaster(ColorModel colorModel, Raster raster) {
        if (raster == null) {
            return;
        }
        if (TexturePaintContext.xrgbmodel == colorModel) {
            TexturePaintContext.xrgbRasRef = new WeakReference(raster);
        }
        else if (TexturePaintContext.argbmodel == colorModel) {
            TexturePaintContext.argbRasRef = new WeakReference(raster);
        }
        return;
    }
    
    static synchronized WritableRaster makeByteRaster(Raster raster, int w, int h) {
        if (TexturePaintContext.byteRasRef != null) {
            WritableRaster writableRaster = (WritableRaster)TexturePaintContext.byteRasRef.get();
            if (writableRaster != null && writableRaster.getWidth() >= w && writableRaster.getHeight() >= h) {
                TexturePaintContext.byteRasRef = null;
                return writableRaster;
            }
        }
        if (w <= 32 && h <= 32) {
            h = (w = 32);
        }
        return raster.createCompatibleWritableRaster(w, h);
    }
    
    static synchronized void dropByteRaster(Raster referent) {
        if (referent == null) {
            return;
        }
        TexturePaintContext.byteRasRef = new WeakReference(referent);
        return;
    }
    
    public abstract WritableRaster makeRaster(int p0, int p1);
    
    public abstract void setRaster(int p0, int p1, int p2, int p3, int p4, int p5, int p6, int p7, int p8, int p9, int p10, int p11, int p12, int p13, int p14, int p15);
    
    public static int blend(int[] array, int n, int n2) {
        n >>>= 19;
        n2 >>>= 19;
        int n6;
        int n5;
        int n4;
        int n3 = n4 = (n5 = (n6 = 0));
        for (int i = 0; i < 4; ++i) {
            int n7 = array[i];
            n = 4096 - n;
            if ((i & 0x1) == 0x0) {
                n2 = 4096 - n2;
            }
            int n8 = n * n2;
            if (n8 != 0) {
                n4 += (n7 >>> 24) * n8;
                n3 += (n7 >>> 16 & 0xFF) * n8;
                n5 += (n7 >>> 8 & 0xFF) * n8;
                n6 += (n7 & 0xFF) * n8;
            }
        }
        return n4 + 8388608 >>> 24 << 24 | n3 + 8388608 >>> 24 << 16 | n5 + 8388608 >>> 24 << 8 | n6 + 8388608 >>> 24;
    }
    
    static {
        TexturePaintContext.xrgbmodel = new DirectColorModel(24, 16711680, 65280, 255);
        TexturePaintContext.argbmodel = ColorModel.getRGBdefault();
    }
    
    static class Int extends TexturePaintContext
    {
        IntegerInterleavedRaster srcRas;
        int[] inData;
        int inOff;
        int inSpan;
        int[] outData;
        int outOff;
        int outSpan;
        boolean filter;
        
        public Int(IntegerInterleavedRaster srcRas, ColorModel colorModel, AffineTransform affineTransform, int n, boolean filter) {
            super(colorModel, affineTransform, srcRas.getWidth(), srcRas.getHeight(), n);
            this.srcRas = srcRas;
            this.inData = srcRas.getDataStorage();
            this.inSpan = srcRas.getScanlineStride();
            this.inOff = srcRas.getDataOffset(0);
            this.filter = filter;
        }
        
        @Override
        public WritableRaster makeRaster(int n, int n2) {
            WritableRaster raster = TexturePaintContext.makeRaster(this.colorModel, this.srcRas, n, n2);
            IntegerInterleavedRaster integerInterleavedRaster = (IntegerInterleavedRaster)raster;
            this.outData = integerInterleavedRaster.getDataStorage();
            this.outSpan = integerInterleavedRaster.getScanlineStride();
            this.outOff = integerInterleavedRaster.getDataOffset(0);
            return raster;
        }
        
        @Override
        public void setRaster(int n, int n2, int n3, int n4, int i, int n5, int n6, int n7, int n8, int n9, int n10, int n11, int n12, int n13, int n14, int n15) {
            int[] inData = this.inData;
            int[] outData = this.outData;
            int outOff = this.outOff;
            int inSpan = this.inSpan;
            int inOff = this.inOff;
            int outSpan = this.outSpan;
            int filter = this.filter;
            int n16 = n8 == 1 && n9 == 0 && n10 == 0 && n11 == 0 && filter == 0;
            if (!(n16 == 0)) {
                outSpan -= i;
            }
            int[] array = filter ? new int[4] : null;
            for (int j = 0; j < n5; outOff += outSpan, ++j) {
                if (!(n16 == 0)) {
                    int n19 = inOff + n2 * inSpan + n6;
                    n = n6 - n;
                    outOff += i;
                    if (n6 >= 32) {
                        while (i > 0) {
                            n = ((i < n) ? i : n);
                            System.arraycopy(inData, n19 - n, outData, outOff - i, n);
                            i -= n;
                            if ((n -= n) == 0) {
                                n = n6;
                            }
                            else {
                                continue;
                            }
                        }
                    }
                    else {
                        for (; i > 0; --i) {
                            outData[outOff - i] = inData[n19 - n];
                            if (--n == 0) {
                                n = n6;
                            }
                        }
                    }
                }
                else {
                    for (int k = 0; k < i; ++k) {
                        if (!(filter == 0)) {
                            int n17;
                            if ((n17 = n + 1) >= n6) {
                                n17 = 0;
                            }
                            int n18;
                            if ((n18 = n2 + 1) >= n7) {
                                n18 = 0;
                            }
                            array[0] = inData[inOff + n2 * inSpan + n];
                            array[1] = inData[inOff + n2 * inSpan + n17];
                            array[2] = inData[inOff + n18 * inSpan + n];
                            array[3] = inData[inOff + n18 * inSpan + n17];
                            outData[outOff + k] = TexturePaintContext.blend(array, n3, n4);
                        }
                        else {
                            outData[outOff + k] = inData[inOff + n2 * inSpan + n];
                        }
                        if ((n3 += n9) < 0) {
                            n3 &= Integer.MAX_VALUE;
                            ++n;
                        }
                        if ((n += n8) >= n6) {
                            n -= n6;
                        }
                        if ((n4 += n11) < 0) {
                            n4 &= Integer.MAX_VALUE;
                            ++n2;
                        }
                        if ((n2 += n10) >= n7) {
                            n2 -= n7;
                        }
                    }
                }
                if ((n3 += n13) < 0) {
                    n3 &= Integer.MAX_VALUE;
                    ++n;
                }
                if ((n += n12) >= n6) {
                    n -= n6;
                }
                if ((n4 += n15) < 0) {
                    n4 &= Integer.MAX_VALUE;
                    ++n2;
                }
                if ((n2 += n14) >= n7) {
                    n2 -= n7;
                }
            }
        }
    }
    
    static class Byte extends TexturePaintContext
    {
        ByteInterleavedRaster srcRas;
        byte[] inData;
        int inOff;
        int inSpan;
        byte[] outData;
        int outOff;
        int outSpan;
        
        public Byte(ByteInterleavedRaster srcRas, ColorModel colorModel, AffineTransform affineTransform, int n) {
            super(colorModel, affineTransform, srcRas.getWidth(), srcRas.getHeight(), n);
            this.srcRas = srcRas;
            this.inData = srcRas.getDataStorage();
            this.inSpan = srcRas.getScanlineStride();
            this.inOff = srcRas.getDataOffset(0);
        }
        
        @Override
        public WritableRaster makeRaster(int n, int n2) {
            WritableRaster byteRaster = TexturePaintContext.makeByteRaster(this.srcRas, n, n2);
            ByteInterleavedRaster byteInterleavedRaster = (ByteInterleavedRaster)byteRaster;
            this.outData = byteInterleavedRaster.getDataStorage();
            this.outSpan = byteInterleavedRaster.getScanlineStride();
            this.outOff = byteInterleavedRaster.getDataOffset(0);
            return byteRaster;
        }
        
        @Override
        public void dispose() {
            TexturePaintContext.dropByteRaster(this.outRas);
        }
        
        @Override
        public void setRaster(int n, int n2, int n3, int n4, int i, int n5, int n6, int n7, int n8, int n9, int n10, int n11, int n12, int n13, int n14, int n15) {
            byte[] inData = this.inData;
            byte[] outData = this.outData;
            int outOff = this.outOff;
            int inSpan = this.inSpan;
            int inOff = this.inOff;
            int outSpan = this.outSpan;
            int n16 = n8 == 1 && n9 == 0 && n10 == 0 && n11 == 0;
            if (!(n16 == 0)) {
                outSpan -= i;
            }
            for (int j = 0; j < n5; outOff += outSpan, ++j) {
                if (!(n16 == 0)) {
                    int n17 = inOff + n2 * inSpan + n6;
                    n = n6 - n;
                    outOff += i;
                    if (n6 >= 32) {
                        while (i > 0) {
                            n = ((i < n) ? i : n);
                            System.arraycopy(inData, n17 - n, outData, outOff - i, n);
                            i -= n;
                            if ((n -= n) == 0) {
                                n = n6;
                            }
                            else {
                                continue;
                            }
                        }
                    }
                    else {
                        for (; i > 0; --i) {
                            outData[outOff - i] = inData[n17 - n];
                            if (--n == 0) {
                                n = n6;
                            }
                        }
                    }
                }
                else {
                    for (int k = 0; k < i; ++k) {
                        outData[outOff + k] = inData[inOff + n2 * inSpan + n];
                        if ((n3 += n9) < 0) {
                            n3 &= Integer.MAX_VALUE;
                            ++n;
                        }
                        if ((n += n8) >= n6) {
                            n -= n6;
                        }
                        if ((n4 += n11) < 0) {
                            n4 &= Integer.MAX_VALUE;
                            ++n2;
                        }
                        if ((n2 += n10) >= n7) {
                            n2 -= n7;
                        }
                    }
                }
                if ((n3 += n13) < 0) {
                    n3 &= Integer.MAX_VALUE;
                    ++n;
                }
                if ((n += n12) >= n6) {
                    n -= n6;
                }
                if ((n4 += n15) < 0) {
                    n4 &= Integer.MAX_VALUE;
                    ++n2;
                }
                if ((n2 += n14) >= n7) {
                    n2 -= n7;
                }
            }
        }
    }
    
    static class ByteFilter extends TexturePaintContext
    {
        ByteInterleavedRaster srcRas;
        int[] inPalette;
        byte[] inData;
        int inOff;
        int inSpan;
        int[] outData;
        int outOff;
        int outSpan;
        
        public ByteFilter(ByteInterleavedRaster srcRas, ColorModel colorModel, AffineTransform affineTransform, int n) {
            super((colorModel.getTransparency() == 1) ? TexturePaintContext.ByteFilter.xrgbmodel : TexturePaintContext.ByteFilter.argbmodel, affineTransform, srcRas.getWidth(), srcRas.getHeight(), n);
            this.inPalette = new int[256];
            ((IndexColorModel)colorModel).getRGBs(this.inPalette);
            this.srcRas = srcRas;
            this.inData = srcRas.getDataStorage();
            this.inSpan = srcRas.getScanlineStride();
            this.inOff = srcRas.getDataOffset(0);
        }
        
        @Override
        public WritableRaster makeRaster(int n, int n2) {
            WritableRaster raster = TexturePaintContext.makeRaster(this.colorModel, null, n, n2);
            IntegerInterleavedRaster integerInterleavedRaster = (IntegerInterleavedRaster)raster;
            this.outData = integerInterleavedRaster.getDataStorage();
            this.outSpan = integerInterleavedRaster.getScanlineStride();
            this.outOff = integerInterleavedRaster.getDataOffset(0);
            return raster;
        }
        
        @Override
        public void setRaster(int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int n9, int n10, int n11, int n12, int n13, int n14, int n15, int n16) {
            byte[] inData = this.inData;
            int[] outData = this.outData;
            int outOff = this.outOff;
            int inSpan = this.inSpan;
            int inOff = this.inOff;
            int outSpan = this.outSpan;
            int[] array = new int[4];
            for (int i = 0; i < n6; outOff += outSpan, ++i) {
                for (int j = 0; j < n5; ++j) {
                    int n17;
                    if ((n17 = n + 1) >= n7) {
                        n17 = 0;
                    }
                    int n18;
                    if ((n18 = n2 + 1) >= n8) {
                        n18 = 0;
                    }
                    array[0] = this.inPalette[0xFF & inData[inOff + n + inSpan * n2]];
                    array[1] = this.inPalette[0xFF & inData[inOff + n17 + inSpan * n2]];
                    array[2] = this.inPalette[0xFF & inData[inOff + n + inSpan * n18]];
                    array[3] = this.inPalette[0xFF & inData[inOff + n17 + inSpan * n18]];
                    outData[outOff + j] = TexturePaintContext.blend(array, n3, n4);
                    if ((n3 += n10) < 0) {
                        n3 &= Integer.MAX_VALUE;
                        ++n;
                    }
                    if ((n += n9) >= n7) {
                        n -= n7;
                    }
                    if ((n4 += n12) < 0) {
                        n4 &= Integer.MAX_VALUE;
                        ++n2;
                    }
                    if ((n2 += n11) >= n8) {
                        n2 -= n8;
                    }
                }
                if ((n3 += n14) < 0) {
                    n3 &= Integer.MAX_VALUE;
                    ++n;
                }
                if ((n += n13) >= n7) {
                    n -= n7;
                }
                if ((n4 += n16) < 0) {
                    n4 &= Integer.MAX_VALUE;
                    ++n2;
                }
                if ((n2 += n15) >= n8) {
                    n2 -= n8;
                }
            }
        }
    }
    
    static class Any extends TexturePaintContext
    {
        WritableRaster srcRas;
        boolean filter;
        
        public Any(WritableRaster srcRas, ColorModel colorModel, AffineTransform affineTransform, int n, boolean filter) {
            super(colorModel, affineTransform, srcRas.getWidth(), srcRas.getHeight(), n);
            this.srcRas = srcRas;
            this.filter = filter;
        }
        
        @Override
        public WritableRaster makeRaster(int n, int n2) {
            return TexturePaintContext.makeRaster(this.colorModel, this.srcRas, n, n2);
        }
        
        @Override
        public void setRaster(int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int n9, int n10, int n11, int n12, int n13, int n14, int n15, int n16) {
            Object o = null;
            WritableRaster srcRas = this.srcRas;
            WritableRaster outRas = this.outRas;
            int[] array = this.filter ? new int[4] : null;
            for (int i = 0; i < n6; ++i) {
                for (int j = 0; j < n5; ++j) {
                    o = srcRas.getDataElements(n, n2, o);
                    if (this.filter) {
                        int n17;
                        if ((n17 = n + 1) >= n7) {
                            n17 = 0;
                        }
                        int n18;
                        if ((n18 = n2 + 1) >= n8) {
                            n18 = 0;
                        }
                        array[0] = this.colorModel.getRGB(o);
                        Object dataElements = srcRas.getDataElements(n17, n2, o);
                        array[1] = this.colorModel.getRGB(dataElements);
                        Object dataElements2 = srcRas.getDataElements(n, n18, dataElements);
                        array[2] = this.colorModel.getRGB(dataElements2);
                        Object dataElements3 = srcRas.getDataElements(n17, n18, dataElements2);
                        array[3] = this.colorModel.getRGB(dataElements3);
                        o = this.colorModel.getDataElements(TexturePaintContext.blend(array, n3, n4), dataElements3);
                    }
                    outRas.setDataElements(j, i, o);
                    if ((n3 += n10) < 0) {
                        n3 &= Integer.MAX_VALUE;
                        ++n;
                    }
                    if ((n += n9) >= n7) {
                        n -= n7;
                    }
                    if ((n4 += n12) < 0) {
                        n4 &= Integer.MAX_VALUE;
                        ++n2;
                    }
                    if ((n2 += n11) >= n8) {
                        n2 -= n8;
                    }
                }
                if ((n3 += n14) < 0) {
                    n3 &= Integer.MAX_VALUE;
                    ++n;
                }
                if ((n += n13) >= n7) {
                    n -= n7;
                }
                if ((n4 += n16) < 0) {
                    n4 &= Integer.MAX_VALUE;
                    ++n2;
                }
                if ((n2 += n15) >= n8) {
                    n2 -= n8;
                }
            }
        }
    }
}
